import { Request, Router } from "express";
import { AuthorizationMode, AuthorizationChecker } from "hyrest";
export declare type ContextFactory<T> = (request: Request) => T | Promise<T>;
export declare type HyrestMiddleware<T> = Router & HyrestBuilder<T>;
export interface HyrestBuilder<T> {
    /**
     * Set a context or context factory on the Hyrest middleware which will be invoked for every call
     * to any endpoint, providing a custom context object.
     */
    context: (contextFactory: ContextFactory<T> | T) => HyrestMiddleware<T>;
    /**
     * Provide a handler checking if the call to an endpoint was authorized.
     */
    authorization: (handler: AuthorizationChecker<T, Request>) => HyrestMiddleware<T>;
    /**
     * Set the default authorization for all routes.
     */
    defaultAuthorizationMode: (mode: AuthorizationMode) => HyrestMiddleware<T>;
}
/**
 * A middleware to use with express. Takes a list of controllers as arguments. All controllers will be attached
 * to react and will receive requests.
 *
 * **Example:**
 *
 * ```
 * app.use(resRpc(new ControllerOne(), new ControllerTwo(), ...));
 * ```
 *
 * @param controllerObjects The instances of controllers to pass express's requests to.
 *
 * @return An express router.
 */
export declare function hyrest<TContext>(...controllerObjects: any[]): HyrestMiddleware<TContext>;
